package gobuf

import (
	"bytes"
	"github.com/emicklei/proto"
	"gitter.top/sync/proto-contrib/pkg/protofmt"
	"io"
	"os"
)

type Parser struct {
	filename string
	rawData  io.Reader
	proto    *proto.Proto
}

func NewParser(file string) (*Parser, error) {
	reader, err := os.Open(file)
	if err != nil {
		return nil, err
	}
	defer reader.Close()

	parser := proto.NewParser(reader)
	definition, err := parser.Parse()
	if err != nil {
		return nil, err
	}
	return &Parser{
		filename: file,
		rawData:  reader,
		proto:    definition,
	}, nil
}

func (parser *Parser) ExistService(serviceName string) bool {
	var result bool
	proto.Walk(parser.proto, proto.WithService(func(service *proto.Service) {
		if service.Name == serviceName {
			result = true
		}
	}))
	return result
}

func (parser *Parser) ExistMessage(messageName string) bool {
	var result bool
	proto.Walk(parser.proto, proto.WithMessage(func(message *proto.Message) {
		if message.Name == messageName {
			result = true
		}
	}))
	return result
}

func (parser *Parser) ExistRPC(serviceName, rpcName string) bool {
	var result bool
	proto.Walk(parser.proto, proto.WithService(func(service *proto.Service) {
		if service.Name != serviceName {
			return
		}
		for _, element := range service.Elements {
			if rpc, ok := element.(*proto.RPC); ok {
				if rpc.Name == rpcName {
					result = true
					return
				}
			}
		}
	}))
	return result
}

func (parser *Parser) AddRPC(serviceName, rpcName string) error {
	proto.Walk(parser.proto, proto.WithService(func(service *proto.Service) {
		if service.Name != serviceName {
			return
		}
		service.Elements = append(service.Elements, &proto.RPC{
			Comment: &proto.Comment{
				Lines: []string{
					" @desc: ",
					" @author: ",
					" @method: ",
					" @api: /" + calm2Case(rpcName),
				},
			},
			Name:        rpcName,
			RequestType: rpcName + "Req",
			ReturnsType: rpcName + "Resp",
			Parent:      service,
		})
	}))

	parser.proto.Elements = append(parser.proto.Elements, &proto.Message{
		Name:   rpcName + "Req",
		Parent: parser.proto,
	})
	parser.proto.Elements = append(parser.proto.Elements, &proto.Message{
		Name:   rpcName + "Resp",
		Parent: parser.proto,
	})

	return parser.writeSync()
}

func (parser *Parser) writeSync() error {
	var buf = new(bytes.Buffer)

	protofmt.NewFormatter(buf, "    ").Format(parser.proto) // 1 tab

	// write back to input
	if err := os.WriteFile(parser.filename, buf.Bytes(), os.ModePerm); err != nil {
		return err
	}

	return nil
}
